@@ -406,4 +406,38 @@ def advcl():
 		if i > 1 : 
 			dic[s] = cursor_rows(f"select lex, count(*) cnt from dic where type='tok' and tag='VBG' and glem='{s}' and dep='advcl' and gpos='VERB' group by lex order by cnt desc")
 	save(dic, "advcl-VERB-VBG.json")
+
+
+function addpat(s){
+    return s.startsWith("_") ? `[^ ]*${s}[^ ]*` : `${s}_[^ ]*`
+}
+
+function rehyb(hyb){ // rehyb   = lambda hyb: ' '.join([ addpat(s) for s in hyb.split()])
+    var arr= []
+    for (var s of hyb.split(' ')) arr.push(addpat(s))
+    return arr.join(' ')
+}
+
+function heads(chunk){ //heads   = lambda chunk:  ' '.join([s.split('_')[0].lower() for s in chunk.split()])		#the_the_DT_DET adventures_adventure_NNS_NOUN of_of_IN_ADP
+	var arr = []
+    for ( var s of chunk.split(' ')) arr.push(s.split('_')[0].lower())
+    return arr.join(' ')
+}
+
+// ǷǺϷ URL
+function isUrl(url) {
+    const regex = /\b(https?):\/\/[\-A-Za-z0-9+&@#\/%?=~_|!:,.;]*[\-A-Za-z0-9+&@#\/%=~_|]/i;
+    return regex.test(url);
+}
+    // Դ
+let url = "https://www.wetools.com";
+let r = isUrl(url);
+
+console.log(rehyb("_one _two"));
+
+var s='yes the_the_DT_DET adventures_adventure_NNS_NOUN of_of_IN_ADP'
+var hyb = 'the_[^ ]* [^ ]*_NNS_[^ ]* of_[^ ]*'
+console.log(s.search(hyb))
+==> 4 
+
 '''
\ No newline at end of file
