@@ -150,6 +150,16 @@ def parse(snt, merge_np= False):
 	'morph': [ t.morph for t in doc],
 	}) 
 
+def show(doc):
+	''' used in the notebook, for debug '''
+	import pandas as pd
+	return pd.DataFrame({'word': [t.text for t in doc], 'tag': [t.tag_ for t in doc],'pos': [t.pos_ for t in doc],'head': [t.head.orth_ for t in doc],'dep': [t.dep_ for t in doc], 'lemma': [t.text.lower() if t.lemma_ == '-PRON-' else t.lemma_ for t in doc],
+	'n_lefts': [ t.n_lefts for t in doc], 'left_edge': [ t.left_edge.text for t in doc], 
+	'n_rights': [ t.n_rights for t in doc], 'right_edge': [ t.right_edge.text for t in doc],
+	'subtree': str([ list(t.subtree) for t in doc]),'children': str([ list(t.children) for t in doc]),
+	'morph': [ t.morph for t in doc],'ent_type': [ t.ent_type_ for t in doc],
+	})
+
 trp_rel		= lambda t:  f"{t.dep_}_{t.head.pos_}_{t.pos_}"  # dobj_VERB_NOUN
 trp_reverse = set({"amod_NOUN_ADJ","nsubj_VERB_NOUN"})
 trp_tok		= lambda doc, arr:  [ t for t in doc if [ t.dep_, t.head.pos_, t.pos_, t.head.lemma_, t.lemma_ ] == arr ] # arr is exactly 5 list 
@@ -668,6 +678,11 @@ def es_postag(doc):
 	from dic.oneself import oneself 
 	return "_^ " + ' '.join([ f"_{t.pos_}_{t.tag_}" if t.pos_ in ('PROPN','NUM','X','SPACE') else f"{t.text if t.text in ('I') else t.text.lower()}_{t.lemma_}_{t.pos_}_{t.tag_}{oneself.get(t.text.lower(), '')}"  for t in doc]) # uniq by ana #,'PUNCT'
 
+def es_skenp(doc):
+	from dic.oneself import oneself 
+	merge_np(doc) # transform doc , finally to be called 
+	return "_^ " + ' '.join([ f"_{t.lemma_}_NP_{t.dep_}" if t.ent_type_ == 'NP' else f"_{t.pos_}_{t.tag_}_{t.dep_}" if t.pos_ in ('PROPN','NUM','X','SPACE') else f"{t.text if t.text in ('I') else t.text.lower()}_{t.lemma_}_{t.pos_}_{t.tag_}{oneself.get(t.text.lower(), '')}_{t.dep_}"  for t in doc]) # uniq by ana #,'PUNCT'
+
 def kps(doc): 
 	''' doc is of single sent, 2022.9.8 '''
 	_kps = []
@@ -715,6 +730,21 @@ def merge_prt(doc):
 				print ( "merge_prt ex:", e , start, end)
 	return doc
 
+def merged(doc): 
+	if not hasattr(merged, 'matcher'):
+		merged.matcher = Matcher(spacy.nlp.vocab)
+		merged.matcher.add("vprt", [[{"POS":"VERB"}, {"POS":"ADP", "DEP":"prt"}]], greedy ='LONGEST')
+		merged.matcher.add("vtov", [[{"POS":"VERB"},{"LEMMA":"to", "TAG":"TO"}, {"TAG":"VB"}]], greedy ='LONGEST')        
+		merged.matcher.add("vvbg", [[{"POS":"VERB"},{"DEP":"xcomp","TAG":"VBG"}]], greedy ='LONGEST')                
+	with doc.retokenize() as retokenizer:
+		for name, start, end in merged.matcher(doc):
+			try:
+				attrs = {"pos": doc[start].pos, "tag": doc[start].tag, "dep": doc[start].dep, "lemma":"_".join([doc[i].lemma_ if i == start else doc[i].text for i in range(start,end)]), "ent_type": "merged_" + spacy.nlp.vocab[name].text}
+				retokenizer.merge(doc[start : end], attrs=attrs)
+			except Exception as e:
+				print ( "merged ex:", e , start, end)
+	return doc
+
 def depmatch(): #from spacy.matcher import Matcher,DependencyMatcher
 	''' 2023.1.6 '''
 	if not hasattr(depmatch,'matcher'): 
@@ -730,6 +760,8 @@ def depmatch(): #from spacy.matcher import Matcher,DependencyMatcher
 			"xcomp-to": [  {"RIGHT_ID": "v", "RIGHT_ATTRS": {"POS": "VERB"}}, { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object",  "RIGHT_ATTRS": {"DEP": "xcomp"} }, {"LEFT_ID": "object", "REL_OP": ";", "RIGHT_ID": "to", "RIGHT_ATTRS": {"LEMMA": "to"} },],
 			# turn off the light
 			"prt-dobj": [  {"RIGHT_ID": "v","RIGHT_ATTRS": {"POS": "VERB"}},  { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "prt"} }, { "LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object", "RIGHT_ATTRS": {"DEP": "dobj"} }],
+			# Some of the Republican policies have left feminists dismayed and aghast.
+			"dobj-acl":[ {"RIGHT_ID": "v","RIGHT_ATTRS": {"POS": "VERB"}}, { "LEFT_ID": "v","REL_OP": ">","RIGHT_ID": "dobj","RIGHT_ATTRS": {"DEP": "dobj"}},{  "LEFT_ID": "dobj", "REL_OP": ">", "RIGHT_ID": "prep", "RIGHT_ATTRS": {"DEP": "acl"}}],
 			# be happy with
 			"acomp-prep":[ {"RIGHT_ID": "v","RIGHT_ATTRS": {"POS": "VERB"}}, { "LEFT_ID": "v","REL_OP": ">","RIGHT_ID": "acomp","RIGHT_ATTRS": {"DEP": "acomp"}},{  "LEFT_ID": "acomp", "REL_OP": ">", "RIGHT_ID": "prep", "RIGHT_ATTRS": {"DEP": "prep"}}],
 			# be based on
@@ -742,8 +774,10 @@ def depmatch(): #from spacy.matcher import Matcher,DependencyMatcher
 
 [setattr(builtins, k, v) for k, v in globals().items() if not k.startswith("_") and not '.' in k and not hasattr(builtins,k) ] #setattr(builtins, "spacy", spacy)
 if __name__	== '__main__': 
-	doc = spacy.nlp("It is on the brink, and I am very happy, and I leave these books opened.") 
-	print (es_postag(doc), kps(doc)) 
+	doc = spacy.nlp("Some of the Republican policies have left feminists dismayed and aghast.") 
+	for name, ar in depmatch()(doc) : 
+		print(spacy.nlp.vocab[name].text, doc[ar[0]].lemma_, doc[ar[1]].lemma_, doc[ar[2]])
+	print ( es_skenp(doc)) 	
 	#merge_np(doc)
 	#print ( post_np_matcher(doc) ) 
 	#print (parse("I love you."))
