@@ -202,13 +202,17 @@ class ES(object):
 	def keys(self, idxname): [print(resp['_id']) for resp in  helpers.scan(client=self.es, query={"query" : {"match_all" : {}}}, scroll= "10m", index= idxname , timeout="10m") ]
 	def hello(self): print (requests.es)
 
-	def dump(self, idxname): 
+	def dump(self, idxname, outfile=None): # dump to *.gz directly ? 
 		''' python -m so dump gzjc  > gzjc.esjson 
 		# {"_index": "gzjc", "_type": "_doc", "_id": "2897-stype", "_source": {"src": 2897, "tag": "simple_snt", "type": "stype"}}'''
-		for doc in helpers.scan(self.es,query={"query": {"match_all": {}}}, index=idxname):
-			del doc["sort"]
-			del doc["_score"]
-			print (json.dumps(doc))  
+		if outfile is None: outfile = idxname + ".esjson"
+		with open(outfile, 'w') as fw: 
+			for doc in helpers.scan(self.es,query={"query": {"match_all": {}}}, index=idxname):
+				del doc["sort"]
+				del doc["_score"]
+				fw.write(json.dumps(doc) + "\n") # print (json.dumps(doc))  
+		os.system(f"gzip -f -9 {outfile}")
+		print ("finished:", idxname, outfile ) 
 
 	def cos_tsv(self, idxname):
 		''' trp(rel, gov, dep, cnt) '''
@@ -269,10 +273,31 @@ class ES(object):
 		if actions : helpers.bulk(client=self.es,actions=actions, raise_on_error=False)
 		print(">>finished " , infile, idxname )
 
+	def merge(self, src, dest):
+		''' 2022.8.29 '''
+		return requests.post(f"http://{requests.eshost}:{requests.esport}/_reindex?pretty", json={"source": { "index": src }, "dest": {   "index": dest}}).json()
+
 if __name__ == '__main__':
 	fire.Fire(ES)
 
 '''
+ubuntu@es-corpusly-com-105-249:/data$ python -m so merge dic en 
+
+took:                   2521235
+timed_out:              false
+total:                  7997698
+updated:                7997698
+created:                0
+deleted:                0
+batches:                7998
+version_conflicts:      0
+noops:                  0
+retries:                {"bulk": 0, "search": 0}
+throttled_millis:       0
+requests_per_second:    -1.0
+throttled_until_millis: 0
+failures:               []
+
 {"_index": "gzjc", "_type": "_doc", "_id": "2897-stype", "_source": {"src": 2897, "tag": "simple_snt", "type": "stype"}}
 ubuntu@es-corpusly-com-105-249:/data/cikuu/pypi/so$ python __main__.py ssi  --query "select pos, lem, count(*) cnt from gzjc where type = 'tok' and lem rlike '[a-z]+' group by pos, lem" > gzjc.poslemcnt
 
