@@ -1,7 +1,45 @@
 #!/usr/bin/env python
-import redis ,fire,json, fileinput #ln -s /home/cikuu/cikuu/bin/redisdl.py redisdl
+import redis ,fire,json, traceback, fileinput #ln -s /home/cikuu/cikuu/bin/redisdl.py redisdl
 
-def dump(outfile, pattern=None, host = '172.17.0.1', port=6379, db=0, password=None):
+def dump(port,pattern:str='*', host='172.17.0.1', db=0, password=None):
+	''' dump by pattern, ie: *  '''
+	r = redis.Redis(host=host, port=port, db=db, decode_responses=True)
+	for k in r.keys(pattern): #r.scan_iter() if not pattern else
+		try:
+			type =  r.type(k)
+			value = r.hgetall(k) if type == 'hash' else dict(r.zrevrange(k, 0,-1, True) ) if type =='zset' else r.lrange(k, 0,-1) if type =='list' else r.smembers(k) if type == 'set' else r.get(k) if type == 'string' else None
+			if value is not None : print ( json.dumps({"key":k, "type":type, "value":value}))
+		except Exception as e: 
+			pass 
+
+def load(infile, host = '172.17.0.1',port=6379,  db=0):
+	''' redisdl load penly-db-config.ktv --db 3 '''
+	r = redis.Redis(host=host, port=port, db=db, decode_responses=True)
+	print (infile, r, flush=True)
+	for line in open(infile,'r',encoding='UTF-8').readlines():
+		try:
+			arr = json.loads(line.strip()) 
+			k,type,v = arr.get('key',''), arr.get('type',''), arr.get('value','')
+			r.delete(k) 
+			if type == 'hash': 
+				r.hset(k, mapping=json.loads(v))
+			elif type == 'zset': 
+				r.zadd(k, json.loads(v))
+			elif type == 'list': 
+				for s in json.loads(v):
+					r.rpush(k, s)
+			elif type == 'set': 
+				for s in json.loads(v):
+					r.sadd(k, s)
+			elif type == 'string': 
+				r.set(k, json.loads(v)) ## \n included 
+		except Exception as e: 
+			print ("ex:", e, "\t|",  line) 
+			traceback.print_exc()
+
+	print ("finished loading:", infile, r)
+
+def dumpktv(outfile, pattern=None, host = '172.17.0.1', port=6379, db=0, password=None):
 	''' dump to db-0.ktv, (key, type, value) '''
 	r = redis.Redis(host=host, port=port, db=db, password=password, decode_responses=True) if password else redis.Redis(host=host, port=port, db=db, decode_responses=True)
 	print (r, outfile, flush=True)
@@ -30,7 +68,7 @@ def keys(pattern, type='string', host = 'localhost', port=6379, db=0):
 		if type == r.type(k): 
 			print(k)
 
-def load(infile, host = '172.17.0.1',port=6379,  db=0, password=None):
+def loadktv(infile, host = '172.17.0.1',port=6379,  db=0, password=None):
 	''' redisdl load penly-db-config.ktv --db 3 '''
 	r = redis.Redis(host=host, port=port, db=db, password=password, decode_responses=True) if password else redis.Redis(host=host, port=port, db=db, decode_responses=True)
 	print (infile, r, flush=True)
